#include "gg_config.h"

gg_config::gg_config()
{

    int r = 0;
    /*
       mode_t tmp_umask = umask(0777);
       umask(tmp_umask);
     */

    xmlNodePtr root_node, current_node;
    xmlDocPtr doc;
    char *config_file;

    char *tmp_content;

    config_file = get_config_filename();

    if (access(config_file, F_OK))
    {

        fprintf(stderr, "Config File Not Present, Loading Defaults.\n");

        gamess_executable = strdup("/usr/local/gamess/rungms");
        gamess_data = g_strconcat(g_get_home_dir(), "/scr/", NULL);
        gamess_scratch = g_strconcat("/scr/", g_get_user_name(), "/", NULL);
        gamess_overwrite = 0;
        gamess_log = 1;
        gamess_options = strdup("");

    }
    else
    {
        doc = xmlParseFile(config_file);

        root_node = xmlDocGetRootElement(doc);

        current_node = root_node->children;

        while (current_node != NULL)
        {
            if (!xmlStrcasecmp(current_node->name, xmlCharStrdup("executable")))
            {
                gamess_executable = (char *) xmlNodeGetContent(current_node);
            }
            else if (!xmlStrcasecmp(current_node->name, xmlCharStrdup("data")))
            {
                gamess_data = (char *) xmlNodeGetContent(current_node);
            }
            else if (!xmlStrcasecmp
                     (current_node->name, xmlCharStrdup("scratch")))
            {
                gamess_scratch = (char *) xmlNodeGetContent(current_node);
            }
            else if (!xmlStrcasecmp
                     (current_node->name, xmlCharStrdup("overwrite")))
            {
                tmp_content = (char *) xmlNodeGetContent(current_node);
                gamess_overwrite = !strcasecmp(tmp_content, "true");
                free(tmp_content);
            }
            else if (!xmlStrcasecmp(current_node->name, xmlCharStrdup("log")))
            {
                tmp_content = (char *) xmlNodeGetContent(current_node);
                gamess_log = !strcasecmp(tmp_content, "true");
                free(tmp_content);
            }
            else if (!xmlStrcasecmp
                     (current_node->name, xmlCharStrdup("options")))
            {
                gamess_options = (char *) xmlNodeGetContent(current_node);
            }

            current_node = current_node->next;
        }
    }

    free(config_file);

}

gg_config::~gg_config()
{

    xmlNodePtr root_node;
    xmlDocPtr doc;

    char *config_file;

    config_file = get_config_filename();

    LIBXML_TEST_VERSION;

    doc = xmlNewDoc(xmlCharStrdup("1.0"));

    root_node = xmlNewNode(NULL, xmlCharStrdup("gtk-gamess"));

    xmlDocSetRootElement(doc, root_node);

    xmlNewChild(root_node, NULL, xmlCharStrdup("executable"),
                xmlCharStrdup(gamess_executable));
    xmlNewChild(root_node, NULL, xmlCharStrdup("data"),
                xmlCharStrdup(gamess_data));
    xmlNewChild(root_node, NULL, xmlCharStrdup("scratch"),
                xmlCharStrdup(gamess_scratch));
    xmlNewChild(root_node, NULL, xmlCharStrdup("options"),
                xmlCharStrdup(gamess_options));
    xmlNewChild(root_node, NULL, xmlCharStrdup("log"),
                xmlCharStrdup(gamess_log ? "true" : "false"));
    xmlNewChild(root_node, NULL, xmlCharStrdup("overwrite"),
                xmlCharStrdup(gamess_overwrite ? "true" : "false"));

    xmlSaveFormatFileEnc(config_file, doc, "UTF-8", 1);

    free(config_file);

    free(gamess_executable);
    free(gamess_options);
    free(gamess_data);
    free(gamess_scratch);

}

char *gg_config::get_config_filename()
{

    int r;

    char *config_file;
    char *config_dir;

    char *tmp_cwd = (char *) malloc(sizeof(char) * PATH_MAX);

    getcwd(tmp_cwd, PATH_MAX);

    config_dir = g_strconcat(g_get_home_dir(), "/.gtk-gamess", NULL);

    r = chdir(config_dir);
    chdir(tmp_cwd);

    if (r != 0)
    {
        fprintf(stderr, "Creating Config Directory: %s\n", config_dir);
        if (mkdir(config_dir, 0755))
        {
            fprintf(stderr, "Error Creating Config Directory.\n");
        }
    }

    config_file = g_strconcat(config_dir, "/config", NULL);

    free(config_dir);

    return (config_file);

}

void gg_config::set_options(char *value)
{
    char *tmp;

    tmp = strdup(value);

    free(gamess_options);

    gamess_options = tmp;
}

void gg_config::set_data(char *value)
{
    char *tmp;

    tmp = strdup(value);

    free(gamess_data);

    gamess_data = tmp;
}

void gg_config::set_scratch(char *value)
{
    char *tmp;

    tmp = strdup(value);

    free(gamess_scratch);

    gamess_scratch = tmp;
}

void gg_config::set_executable(char *value)
{
    char *tmp;

    tmp = strdup(value);

    free(gamess_executable);

    gamess_executable = tmp;
}

void gg_config::set_log(int value)
{
    gamess_log = value;
}

void gg_config::set_overwrite(int value)
{
    gamess_overwrite = value;
}

char *gg_config::get_options()
{
    return (strdup(gamess_options));
}

char *gg_config::get_data()
{
    return (strdup(gamess_data));
}

char *gg_config::get_scratch()
{
    return (strdup(gamess_scratch));
}

char *gg_config::get_executable()
{
    return (strdup(gamess_executable));
}

int gg_config::get_log()
{
    return (gamess_log);
}

int gg_config::get_overwrite()
{
    return (gamess_overwrite);
}
