#include "gg_main_window.h"

using namespace std;

gg_main_window::gg_main_window(gg_config * config):
num_tasks(0)
{
    guint timer_func;
    struct sockaddr_un socket_address;
    int socket_flags;
    char *tmp_title;
    GtkTreeViewColumn *col;
    GtkCellRenderer *renderer;

    current_iter.stamp = 0;

    if ((server_socket = socket(AF_UNIX, SOCK_STREAM, 0)) == -1)
    {
        perror("Cannot Create Server Socket");
        exit(2);
    }

    socket_flags = fcntl(server_socket, F_GETFL);
    fcntl(server_socket, F_SETFL, socket_flags | O_NONBLOCK);

    socket_path =
        g_strconcat(g_get_tmp_dir(), "/", PACKAGE_NAME, "-",
                    g_get_user_name(), NULL);

    if ((sizeof(char) * strlen(socket_path)) >
        (sizeof(socket_address.sun_path) - 1))
    {
        printf
            ("Socket file path is too long please update your temp directory location:\n%s\n",
             socket_path);
        exit(3);
    }

    bzero(&socket_address, sizeof(socket_address));
    socket_address.sun_family = AF_UNIX;
    strcpy(socket_address.sun_path, socket_path);
#ifdef HAVE_SOCKADDR_UN_SUN_LEN
    /*
     * not sure what to set this to 
     */
    /*
     * socket_address.sun_len = strlen(socket_address.sun_path); 
     */
    socket_address.sun_len = SUN_LEN(&socket_address);
#endif
    unlink(socket_path);

    if (bind
        (server_socket, (struct sockaddr *) &socket_address,
         SUN_LEN(&socket_address)) == -1)
    {
        perror("Error Binding to Server Socket");
        exit(2);
    }

    if (listen(server_socket, 5) == -1)
    {
        perror("Error Listening on Server Socket");
        exit(2);
    }

    FD_ZERO(&socket_set);
    FD_SET(server_socket, &socket_set);
    max_socket = server_socket;

    /*
     * open the pipe for reading 
     */
    /*
     * if( (mkfifo(fifo_path, FILE_MODE) < 0) && (errno != EEXIST) ) {
     * perror("Cannot Create Fifo"); exit(2); }
     * 
     * if( (fifo = open( fifo_path, O_RDONLY | O_NONBLOCK, 0 )) < 0 ) {
     * perror("Cannot Open Fifo"); exit(2); } 
     */

    this->config = config;

    xml =
        glade_xml_new(g_strconcat
                      (GLADE_DATA, "gg_main_window.glade", NULL), NULL, NULL);

    if (xml == NULL)
    {
        printf("Failed to Load Glade Files from %s\n", GLADE_DATA);
        xml =
            glade_xml_new(g_strconcat
                          (GLADE_DATA_SRC, "gg_main_window.glade", NULL),
                          NULL, NULL);

        if (xml == NULL)
        {
            printf("Failed to Load Glade Files from %s\n", GLADE_DATA_SRC);

            exit(4);

        }
    }

    widget = glade_xml_get_widget(xml, "main_window");
    statusbar = glade_xml_get_widget(xml, "statusbar");

    tmp_title = g_strconcat(APP_NAME, " ", VERSION, NULL);

    gtk_window_set_title((GtkWindow *) widget, tmp_title);

    free(tmp_title);

    timer_func =
        g_timeout_add(1000, (GSourceFunc) handler_child_timer, (gpointer) this);

    // glade_xml_signal_connect_data(xml, "on_task_list_select_row",
    // (GtkSignalFunc) handler_select_row, (gpointer) this );
    glade_xml_signal_connect_data(xml, "on_window_main_destroy",
                                  (GtkSignalFunc) handler_destroy,
                                  (gpointer) this);
    glade_xml_signal_connect_data(xml, "on_exit_activate",
                                  (GtkSignalFunc) handler_exit,
                                  (gpointer) this);
    glade_xml_signal_connect_data(xml, "on_add_activate",
                                  (GtkSignalFunc) handler_add, (gpointer) this);
    glade_xml_signal_connect_data(xml, "on_preferences_activate",
                                  (GtkSignalFunc) handler_preferences,
                                  (gpointer) this);
    glade_xml_signal_connect_data(xml, "on_button_start_clicked",
                                  (GtkSignalFunc) handler_start,
                                  (gpointer) this);
    glade_xml_signal_connect_data(xml, "on_button_stop_clicked",
                                  (GtkSignalFunc) handler_stop,
                                  (gpointer) this);
    glade_xml_signal_connect_data(xml, "on_button_remove_clicked",
                                  (GtkSignalFunc) handler_remove,
                                  (gpointer) this);
    glade_xml_signal_connect_data(xml, "on_button_requeue_clicked",
                                  (GtkSignalFunc) handler_requeue,
                                  (gpointer) this);

    list = (GtkWidget *) glade_xml_get_widget(xml, "task_list");
    gtk_tree_view_set_reorderable(GTK_TREE_VIEW(list), TRUE);

    // pid
    col = gtk_tree_view_column_new();
    gtk_tree_view_column_set_title(col, "PID");
    gtk_tree_view_append_column(GTK_TREE_VIEW(list), col);
    renderer = gtk_cell_renderer_text_new();
    gtk_tree_view_column_pack_start(col, renderer, TRUE);
    gtk_tree_view_column_add_attribute(col, renderer, "text", COL_PID);

    // name
    col = gtk_tree_view_column_new();
    gtk_tree_view_column_set_title(col, "Name");
    gtk_tree_view_append_column(GTK_TREE_VIEW(list), col);
    renderer = gtk_cell_renderer_text_new();
    gtk_tree_view_column_pack_start(col, renderer, TRUE);
    gtk_tree_view_column_add_attribute(col, renderer, "text", COL_NAME);

    // status
    col = gtk_tree_view_column_new();
    gtk_tree_view_column_set_title(col, "Status");
    gtk_tree_view_append_column(GTK_TREE_VIEW(list), col);
    renderer = gtk_cell_renderer_text_new();
    gtk_tree_view_column_pack_start(col, renderer, TRUE);
    gtk_tree_view_column_add_attribute(col, renderer, "text", COL_STATUS);

    // this is a GtkTreeView now
    list_model = GTK_TREE_MODEL(gtk_list_store_new(NUM_COLS,
                                                   G_TYPE_STRING,
                                                   G_TYPE_STRING,
                                                   G_TYPE_STRING,
                                                   G_TYPE_POINTER));

    gtk_tree_view_set_model(GTK_TREE_VIEW(list), list_model);

    gtk_widget_show(widget);

    msg_warn("Window Initialized\n");

}

/*
 * void gg_main_window::handler_select_row(GtkTreeView *treeview, gboolean 
 * arg1, gpointer data) { gg_main_window * window = (gg_main_window *)
 * data; window->select_row(); }
 * 
 * void gg_main_window::select_row() { GtkTreeSelection *selection;
 * selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(list));
 * gtk_tree_selection_get_selected(selection, &list_model, &selected_row);
 * } 
 */

void gg_main_window::handler_start(GtkWidget * caller, gpointer data)
{
    gg_main_window *window = (gg_main_window *) data;
    window->start_next_task();
}

void gg_main_window::start(gg_task * task)
{
    pid_t child_pid, log_pid;

    int n;
    int write_log;
    int log_pipe[2];
    int log_file;

    char *filename_prefix;
    char *dir, *filename, *executable, *options;
    char *child_pid_str;
    char *data_filename;

    char *log_data, *log_filename;

    if (access(task->filename, R_OK))
    {
        msg_error("Invalid File!");
        fprintf(stderr, "Invalid File: %s\n", task->filename);
        // gtk_clist_set_text(list, row, 2, g_strdup("Invalid File"));
        task->pid = -1;
        return;
    }
    /*
     * get the data before we fork for purposes of consistency 
     */
    write_log = config->get_log();
    dir = g_dirname(task->filename);
    filename = (char *) g_basename(task->filename);
    executable = config->get_executable();
    options = config->get_options();

    filename_prefix = get_filename_prefix(filename);

    data_filename =
        g_strconcat(config->get_data(), "/", filename_prefix, ".dat", NULL);
    if (config->get_overwrite() && !access(data_filename, F_OK))
    {
        unlink(data_filename);
    }
    free(data_filename);

    data_filename =
        g_strconcat(config->get_data(), "/", filename_prefix, ".irc", NULL);
    if (config->get_overwrite() && !access(data_filename, F_OK))
    {
        unlink(data_filename);
    }
    free(data_filename);


    child_pid = fork();
    if (child_pid == 0)
    {                           /* child process */

        /*
         * check if we can change to the directory we need to 
         */
        if (chdir(dir))
        {
            printf("Directory %s does not exist.\n");
            _exit(2);
        }

        free(dir);

        /*
         * going to create a pipe and write the log file if necissary 
         */

        if (write_log)
        {
            if (pipe(log_pipe) < 0)
            {
                printf("Error Creating Pipe.\n");
                _exit(2);
            }

            log_pid = fork();
            if (log_pid == 0)
            {                   /* child process (writes the logfile) */

                close(log_pipe[1]);

                log_data = (char *) malloc(sizeof(char) * 100);

                /*
                 * generate the logfile name 
                 */
                log_filename = g_strconcat(filename_prefix, ".log", NULL);

                if (!access(log_filename, F_OK))
                {
                    unlink(log_filename);
                }

                if ((log_file =
                     open(log_filename, O_CREAT | O_WRONLY, FILE_MODE)) < 0)
                {
                    printf("Error Opening Log File.\n");
                    _exit(2);
                }

                while ((n = read(log_pipe[0], log_data, 100)) > 0)
                {
                    write(log_file, log_data, n);
                }

                close(log_pipe[0]);
                close(log_file);

                free(log_data);
                _exit(0);
            }
            else
            {
                dup2(log_pipe[1], fileno(stdout));
                dup2(log_pipe[1], fileno(stderr));
                close(log_pipe[0]);
                close(log_pipe[1]);
            }
        }

        if (execl(executable, executable, filename, options, NULL) == -1)
        {
            printf("Error Running %s: %s\n", executable, strerror(errno));
        }

        _exit(1);

    }
    else
    {                           /* parent process */

        msg_warn("Queue Started");
        printf("Started Process: %d\n", child_pid);
        task->pid = child_pid;
        task->status = GG_TASK_RUNNING;
    }

    free(dir);
    free(executable);
    free(options);
    free(filename_prefix);

}

void gg_main_window::start_next_task()
{
    gg_task *task;
    gint i = 0;

    GtkTreeSelection *selection;
    GtkTreeIter iter;

    if (current_iter.stamp != 0)
    {
        gtk_tree_model_get(list_model, &current_iter, COL_DATA, &task, -1);
        if (task->status == GG_TASK_RUNNING)
        {
            msg_error("Task already started!\n");
            return;
        }
    }

    if (!gtk_tree_model_get_iter_first(list_model, &current_iter))
    {
        msg_error("No Tasks in Queue!\n");
        return;
    }

    do
    {
        gtk_tree_model_get(list_model, &current_iter, COL_DATA, &task, -1);

        if (task->status == GG_TASK_QUEUED)
        {
            msg_warn("Starting Next Task\n");
            gtk_list_store_set(GTK_LIST_STORE(list_model), &current_iter,
                               COL_STATUS, "Running", -1);
            start(task);
            char pid_str[25];
            snprintf(pid_str, 25, "%d", task->pid);
            gtk_list_store_set(GTK_LIST_STORE(list_model), &current_iter,
                               COL_PID, pid_str, -1);
            return;
        }
    }
    while (gtk_tree_model_iter_next(list_model, &current_iter));

}

void gg_main_window::handler_stop(GtkWidget * caller, gpointer data)
{
    gg_main_window *window = (gg_main_window *) data;
    window->stop_current_task();
}

void gg_main_window::stop_current_task()
{
    gg_task *task;
    if (current_iter.stamp != 0)
    {
        gtk_tree_model_get(list_model, &current_iter, COL_DATA, &task, -1);
        gtk_list_store_set(GTK_LIST_STORE(list_model), &current_iter,
                           COL_STATUS, "Stopping...", -1);
        stop(task);
        gtk_list_store_set(GTK_LIST_STORE(list_model), &current_iter,
                           COL_STATUS, "Stopped", -1);
    }
}

void gg_main_window::stop(gg_task * task)
{
    pid_t child_pid;

    char *tmp_filename;
    char num_string[4];
    char *filename, *filename_prefix;


    int child_status, x;

    if (task->status != GG_TASK_RUNNING)
    {
        msg_error("Task Not Running!\n");
        // return -1;
    }
    else
    {
        filename = (char *) g_path_get_basename(task->filename);

        filename_prefix = get_filename_prefix(filename);

        for (x = 5; x <= 91; x++)
        {
            sprintf(num_string, "%02d", x);

            tmp_filename =
                g_strconcat(config->get_scratch(), "/", filename_prefix,
                            ".F", num_string, NULL);
            unlink(tmp_filename);
            free(tmp_filename);
        }

        kill(task->pid, 1);
        waitpid(task->pid, &child_status, 0);
        // gtk_clist_set_text(list, row, 2, g_strdup("Queued"));
        // gtk_clist_set_text(list, row, 0, g_strdup(""));

        msg_warn("Queue Stopped");
        printf("Stopped Process: %d\n", task->pid);
        task->pid = 0;
        task->status = GG_TASK_STOPPED;

        free(filename);
        free(filename_prefix);
    }

    // return 0;
}


gboolean gg_main_window::handler_child_timer(gpointer data)
{
    gg_main_window *window = (gg_main_window *) data;

    return window->child_timer();

}


gboolean gg_main_window::child_timer()
{
    gg_task *task = NULL;
    int i = 0;
    int disconnect;
    pid_t child_pid;

    char *status_string;
    char *socket_data;
    char *token;

    fd_set temp_set;

    struct sockaddr_un socket_address;

    struct timeval select_wait;

    int data_size, socket_flags;
    int new_socket, socket_address_length;
    int child_status;

    child_pid = waitpid(-1, &child_status, WNOHANG);

    if (child_pid > 0)
    {
        if (current_iter.stamp != 0)
        {
            gtk_tree_model_get(list_model, &current_iter, COL_DATA, &task, -1);

            if (WIFEXITED(child_status))
            {
                switch (WEXITSTATUS(child_status))
                {
                case 4:
                    status_string = "Previously Completed / Invalid";
                    break;
                case 8:
                    status_string = "Invalid Gamess Version";
                    break;
                default:
                    status_string = "Complete";
                };
            }
            else
            {
                status_string = "Error";
            }

            gtk_list_store_set(GTK_LIST_STORE(list_model), &current_iter,
                               COL_STATUS, status_string, -1);
            task->status = GG_TASK_STOPPED;

            start_next_task();

            /*
             * while( (tmp_task = (gg_task*) gtk_clist_get_row_data(list, i))
             * != NULL ) { if( tmp_task->pid == child_pid ) {
             * if(WIFEXITED(child_status)) { switch(WEXITSTATUS(child_status))
             * { case 4: status_string = "Previously Completed / Invalid";
             * break; case 8: status_string = "Invalid Gamess Version"; break;
             * default: status_string = "Complete"; } } else { status_string = 
             * "Error"; };
             * 
             * gtk_clist_set_text(list, i, 2, g_strdup(status_string));
             * tmp_task->pid = -1; current_task = -1;
             * 
             * start_next_task(); break; } else { i++; } }
             * 
             */
        }
    }

    /*
     * make a copy of our set 
     */
    memcpy(&temp_set, &socket_set, sizeof(socket_set));

    /*
     * clear our wait timer 
     */
    memset((char *) &select_wait, 0, sizeof(select_wait));

    if (select(max_socket + 1, &temp_set, NULL, NULL, &select_wait) > 0)
    {

        /*
         * create and clear our read buffer 
         */
        socket_data = (char *) malloc(sizeof(char) * BUFFSIZE);
        memset((char *) socket_data, 0, sizeof(char) * BUFFSIZE);

        if (FD_ISSET(server_socket, &temp_set))
        {
            socket_address_length = sizeof(socket_address);
            if ((new_socket =
                 accept(server_socket,
                        (struct sockaddr *) &socket_address,
                        (socklen_t *) & socket_address_length)) == -1)
            {
                perror("Erorr Accepting Socket");
                exit(2);
            }

            FD_SET(new_socket, &socket_set);

            /*
             * make the new socket non-blocking 
             */
            socket_flags = fcntl(new_socket, F_GETFL);
            fcntl(new_socket, F_SETFL, socket_flags | O_NONBLOCK);

            if (new_socket > max_socket)
            {
                max_socket = new_socket;
            }
            FD_CLR(server_socket, &temp_set);
        }

        for (i = 0; i <= max_socket; i++)
        {
            if (FD_ISSET(i, &temp_set))
            {
                disconnect = 1;

                /*
                 * make sure to read all the data from the socket 
                 */
                while ((data_size = read(i, socket_data, BUFFSIZE)) > 0)
                {
                    token = socket_data;
                    disconnect = 0;

                    while (token < socket_data + data_size)
                    {
                        this->file(token);
                        token = token + strlen(token) + 1;
                    }
                }

                if (disconnect)
                {
                    FD_CLR(i, &socket_set);
                    shutdown(i, 2);
                    close(i);
                }
                else
                {
                    /*
                     * send a response just so the client knows
                     * we've got all its data and it can close.  
                     * again, another cygwin patch!
                     */
                    write(i, ".", 2);
                }
            }
        }
        free(socket_data);
    }

    return TRUE;

}

void gg_main_window::handler_destroy(GtkWidget * caller, gpointer data)
{
    gg_main_window *window = (gg_main_window *) data;
    gtk_main_quit();
}

void gg_main_window::handler_exit(GtkWidget * caller, gpointer data)
{
    gg_main_window *window = (gg_main_window *) data;

    window->destroy_window();

}

void gg_main_window::destroy_window()
{

    gtk_widget_destroy(widget);

}

void gg_main_window::handler_add(GtkWidget * caller, gpointer data)
{

    gg_main_window *window = (gg_main_window *) data;

    window->open_file();
}

void gg_main_window::open_file()
{

    gg_file_selector *selector =
        new gg_file_selector((_file_ok_handler) handler_file,
                             (void *) this);

}

void gg_main_window::handler_preferences(GtkWidget * caller, gpointer data)
{
    gg_main_window *window = (gg_main_window *) data;
    window->preferences();
}

void gg_main_window::preferences()
{
    gg_preferences_window *preferences = new gg_preferences_window(config);
}

void gg_main_window::handler_file(char *filename, void *data)
{
    gg_main_window *window = (gg_main_window *) data;

    window->file(filename);
}

void gg_main_window::file(char *filename)
{
    gint row_number;

    GtkTreeIter iter;

    if (list == NULL)
    {
        printf("Whoops! Could not find task_list.\n");
    }
    else
    {
        gg_task *task = new gg_task((char *) filename);

        gtk_list_store_append(GTK_LIST_STORE(list_model), &iter);
        gtk_list_store_set(GTK_LIST_STORE(list_model), &iter,
                           COL_PID, "",
                           COL_NAME, g_basename(filename),
                           COL_STATUS, "Queued", COL_DATA, task, -1);

        /*
         * gchar **new_entry = g_new0(gchar*, 4); new_entry[0] =
         * g_strdup(""); new_entry[1] = g_strdup(g_basename(filename));
         * new_entry[2] = g_strdup("Queued"); new_entry[3] = NULL; 
         */

        // row_number = gtk_clist_append(list, new_entry);

        // gtk_clist_set_row_data_full(list, row_number, (gpointer) task,
        // (GtkDestroyNotify) gg_task::handler_destroy);

        num_tasks++;

        // g_strfreev(new_entry);

        /*
         * start_task(row_number); 
         */
    }
}

gg_main_window::~gg_main_window(void)
{
    printf("Destroying Window\n");

    close(server_socket);
    unlink(socket_path);
    free(socket_path);

    /*
     * close(fifo); unlink(fifo_path);
     * 
     * free(fifo_path); 
     */
}

void gg_main_window::handler_remove(GtkWidget * caller, gpointer data)
{
    gg_main_window *window = (gg_main_window *) data;
    window->remove();
}

void gg_main_window::remove()
{
    GtkTreeIter iter;
    GtkTreeSelection *selection;
    gg_task *task;

    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(list));
    if (!gtk_tree_selection_get_selected(selection, &list_model, &iter))
    {
        msg_error("No Task Selected!\n");
        return;
    }

    gtk_tree_model_get(list_model, &iter, COL_DATA, &task, -1);

    if (task->status == GG_TASK_RUNNING)
    {
        stop(task);
        start_next_task();
    }

    gtk_list_store_remove(GTK_LIST_STORE(list_model), &iter);

    msg_warn("Task Removed");
    // delete(task);

}

void gg_main_window::handler_requeue(GtkWidget * caller, gpointer data)
{
    gg_main_window *window = (gg_main_window *) data;
    window->requeue();
}

void gg_main_window::requeue()
{
    GtkTreeIter iter;
    GtkTreeSelection *selection;
    gg_task *task;

    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(list));
    if (!gtk_tree_selection_get_selected(selection, &list_model, &iter))
    {
        msg_error("No Task Selected!\n");
        return;
    }

    gtk_tree_model_get(list_model, &iter, COL_DATA, &task, -1);
    task->status = GG_TASK_QUEUED;
    gtk_list_store_set(GTK_LIST_STORE(list_model), &iter, COL_PID, "",
                       COL_STATUS, "Queued", -1);

}

void gg_main_window::msg_error(char *string)
{
    guint context =
        gtk_statusbar_get_context_id(GTK_STATUSBAR(statusbar), "error");
    gtk_statusbar_push(GTK_STATUSBAR(statusbar), context, string);
}

void gg_main_window::msg_warn(char *string)
{
    guint context =
        gtk_statusbar_get_context_id(GTK_STATUSBAR(statusbar), "warning");
    gtk_statusbar_push(GTK_STATUSBAR(statusbar), context, string);
}

char *gg_main_window::get_filename_prefix(char *filename)
{

    char *filename_prefix;
    int i, prefix_length;

    if ((filename_prefix = strrchr(filename, '.')) != NULL)
    {
        prefix_length = filename_prefix - filename;
        filename_prefix = (char *) malloc(sizeof(char) * (prefix_length + 1));

        for (i = 0; i < prefix_length; i++)
        {
            filename_prefix[i] = filename[i];
        }
        filename_prefix[i] = '\0';


    }
    else
    {
        filename_prefix = strdup(filename);
    }

    return (filename_prefix);

}
