#include "gg_preferences_window.h"

gg_preferences_window::gg_preferences_window(gg_config * config)
{
    GtkEntry *entry;
    GtkToggleButton *toggle_button;
    char *value;

    this->xml = xml;
    this->config = config;

    /* xml = glade_xml_new(g_strdup("../glade/gg_preferences_window.glade"), NULL); */

    xml =
        glade_xml_new(g_strconcat
                      (GLADE_DATA, "gg_preferences_window.glade", NULL), NULL,
                      NULL);

    if (xml == NULL)
    {
        printf("Failed to Load Glade Files from %s\n", GLADE_DATA);
        xml =
            glade_xml_new(g_strconcat
                          (GLADE_DATA_SRC, "gg_preferences_window.glade", NULL),
                          NULL, NULL);

        if (xml == NULL)
        {
            printf("Failed to Load Glade Files from %s\n", GLADE_DATA_SRC);

            exit(4);

        }
    }

    widget = glade_xml_get_widget(xml, "preferences_window");

    glade_xml_signal_connect_data(xml, "on_button_executable_clicked",
                                  (GtkSignalFunc) handler_executable,
                                  (gpointer) this);
    glade_xml_signal_connect_data(xml, "on_button_data_clicked",
                                  (GtkSignalFunc) handler_data,
                                  (gpointer) this);
    glade_xml_signal_connect_data(xml, "on_button_scratch_clicked",
                                  (GtkSignalFunc) handler_scratch,
                                  (gpointer) this);
    glade_xml_signal_connect_data(xml, "on_button_save_clicked",
                                  (GtkSignalFunc) handler_save,
                                  (gpointer) this);
    glade_xml_signal_connect_data(xml, "on_button_cancel_clicked",
                                  (GtkSignalFunc) handler_cancel,
                                  (gpointer) this);

    value = config->get_executable();
    entry = (GtkEntry *) glade_xml_get_widget(xml, "entry_executable");
    gtk_entry_set_text(entry, value);
    free(value);

    value = config->get_data();
    entry = (GtkEntry *) glade_xml_get_widget(xml, "entry_data");
    gtk_entry_set_text(entry, value);
    free(value);

    value = config->get_scratch();
    entry = (GtkEntry *) glade_xml_get_widget(xml, "entry_scratch");
    gtk_entry_set_text(entry, value);
    free(value);

    value = config->get_options();
    entry = (GtkEntry *) glade_xml_get_widget(xml, "entry_options");
    gtk_entry_set_text(entry, value);
    free(value);

    toggle_button =
        (GtkToggleButton *) glade_xml_get_widget(xml, "checkbutton_log");
    gtk_toggle_button_set_active(toggle_button, config->get_log());

    toggle_button =
        (GtkToggleButton *) glade_xml_get_widget(xml, "checkbutton_overwrite");
    gtk_toggle_button_set_active(toggle_button, config->get_overwrite());

    gtk_widget_show(widget);
    gtk_window_set_modal((GtkWindow *) widget, TRUE);

}

void gg_preferences_window::handler_cancel(GtkWidget * caller, gpointer data)
{
    gg_preferences_window *preferences = (gg_preferences_window *) data;
    preferences->cancel();
}

void gg_preferences_window::cancel()
{
    gtk_widget_destroy(widget);
}

void gg_preferences_window::handler_save(GtkWidget * caller, gpointer data)
{
    gg_preferences_window *preferences = (gg_preferences_window *) data;
    preferences->save();
}

void gg_preferences_window::save()
{
    GtkEntry *entry;
    GtkToggleButton *toggle_button;
    char *value;

    entry = (GtkEntry *) glade_xml_get_widget(xml, "entry_executable");
    value = gtk_editable_get_chars((GtkEditable *) entry, 0, -1);
    config->set_executable(value);
    free(value);

    entry = (GtkEntry *) glade_xml_get_widget(xml, "entry_data");
    value = gtk_editable_get_chars((GtkEditable *) entry, 0, -1);
    config->set_data(value);
    free(value);

    entry = (GtkEntry *) glade_xml_get_widget(xml, "entry_scratch");
    value = gtk_editable_get_chars((GtkEditable *) entry, 0, -1);
    config->set_scratch(value);
    free(value);

    entry = (GtkEntry *) glade_xml_get_widget(xml, "entry_options");
    value = gtk_editable_get_chars((GtkEditable *) entry, 0, -1);
    config->set_options(value);
    free(value);

    toggle_button =
        (GtkToggleButton *) glade_xml_get_widget(xml, "checkbutton_log");
    config->set_log(gtk_toggle_button_get_active(toggle_button));

    toggle_button =
        (GtkToggleButton *) glade_xml_get_widget(xml, "checkbutton_overwrite");
    config->set_overwrite(gtk_toggle_button_get_active(toggle_button));

    gtk_widget_destroy(widget);
}

void gg_preferences_window::handler_destroy(GtkWidget * caller, gpointer data)
{
    gg_preferences_window *preferences = (gg_preferences_window *) data;
    delete preferences;
}

void gg_preferences_window::handler_executable(GtkWidget * caller,
                                               gpointer data)
{
    gg_file_selector *selector =
        new gg_file_selector((_file_ok_handler) handler_browse_executable,
                             (void *) data);
}

void gg_preferences_window::handler_browse_executable(char *filename,
                                                      void *data)
{
    gg_preferences_window *window = (gg_preferences_window *) data;
    window->browse_executable(filename);
}

void gg_preferences_window::browse_executable(char *filename)
{
    char *file;

    GtkEntry *entry =
        (GtkEntry *) glade_xml_get_widget(xml, "entry_executable");

    file = strdup(filename);

    gtk_entry_set_text(entry, file);

    free(file);

}

void gg_preferences_window::handler_data(GtkWidget * caller, gpointer data)
{
    gg_file_selector *selector =
        new gg_file_selector((_file_ok_handler) handler_browse_data,
                             (void *) data);
}

void gg_preferences_window::handler_browse_data(char *filename, void *data)
{
    gg_preferences_window *window = (gg_preferences_window *) data;
    window->browse_data(filename);
}

void gg_preferences_window::browse_data(char *filename)
{
    char *dir, *full_dir;

    GtkEntry *entry = (GtkEntry *) glade_xml_get_widget(xml, "entry_data");

    dir = g_dirname(filename);
    full_dir = g_strconcat(dir, "/", NULL);

    gtk_entry_set_text(entry, full_dir);

    free(dir);
    free(full_dir);

}

void gg_preferences_window::handler_scratch(GtkWidget * caller, gpointer data)
{
    gg_file_selector *selector =
        new gg_file_selector((_file_ok_handler) handler_browse_scratch,
                             (void *) data);
}

void gg_preferences_window::handler_browse_scratch(char *filename, void *data)
{
    gg_preferences_window *window = (gg_preferences_window *) data;
    window->browse_scratch(filename);
}

void gg_preferences_window::browse_scratch(char *filename)
{
    char *dir, *full_dir;

    GtkEntry *entry = (GtkEntry *) glade_xml_get_widget(xml, "entry_scratch");

    dir = g_dirname(filename);
    full_dir = g_strconcat(dir, "/", NULL);

    gtk_entry_set_text(entry, full_dir);

    free(dir);
    free(full_dir);

}

gg_preferences_window::~gg_preferences_window(void)
{
}
